"""
Zurich Instruments LabOne Python API Example

Demonstrate how to connect to a Zurich Instruments device via the Data Server
program.
"""

# Copyright 2015 Zurich Instruments AG

import re
import numpy as np

import zhinst.utils as utils
import zhinst.ziPython as ziPython


def run_example(device_id):
    """
    Run the example: Connect to a Zurich Instruments device via the Data Server,
    ensure the demodulators are enabled and obtain a single demodulator sample
    via getSample(). Calculate the sample's RMS amplitude and add it as a field
    to the "sample" dictionary.

    Note:

      This is intended to be a simple example demonstrating how to connect to a
      Zurich Instruments device from ziPython. In most cases, data acquisition
      should use either ziDAQServer's poll() method or an instance of the
      ziDAQRecorder class, not the getSample() method.

    Requirements:

      Hardware configuration: Connect signal output 1 to signal input 1 with a
      BNC cable.

    Arguments:

      device_id (str): The ID of the device to run the example with. For
        example, `dev2006` or `uhf-dev2006`.

    Returns:

      sample (dict): The acquired demodulator sample dictionary.

    Raises:

      RuntimeError: If the device is not connected to the Data Server.
    """
    # Create an instance of the ziDiscovery class.
    d = ziPython.ziDiscovery()

    # Determine the device identifier from it's ID.
    device = d.find(device_id).lower()

    # Get the device's default connectivity properties.
    props = d.get(device)

    # The maximum API level supported by this example.
    apilevel_example = 5
    # The maximum API level supported by the device class, e.g., MF.
    apilevel_device = props['apilevel']
    # Ensure we run the example using a supported API level.
    apilevel = min(apilevel_device, apilevel_example)
    # See the LabOne Programming Manual for an explanation of API levels.

    # Create a connection to a Zurich Instruments Data Server (an API session)
    # using the device's default connectivity properties.
    daq = ziPython.ziDAQServer(props['serveraddress'], props['serverport'], apilevel)

    # Check that the device is visible to the Data Server
    if device not in utils.devices(daq):
        raise RuntimeError("The specified device `%s` is not visible to the Data Server, " % device_id + \
                               "please ensure the device is connected by using the LabOne User Interface " + \
                               "or ziControl (HF2 Instruments).")

    # Find out whether the device is an HF2 or a UHF
    devtype = daq.getByte('/%s/features/devtype' % device)
    # If the device is a UHF enable the demodulator
    if re.match('UHF', devtype):
        daq.setInt('/%s/demods/0/enable' % device, 1)
    # Set the demodulator output rate
    daq.setDouble('/%s/demods/0/rate' % device, 1.0e3)

    # Perform a global synchronisation between the device and the data server:
    # Ensure that the settings have taken effect on the device before issuing
    # the getSample() command
    daq.sync()

    # Obtain one demodulator sample
    sample = daq.getSample('/%s/demods/0/sample' % device)
    sample['R'] = np.sqrt(sample['x']**2 + sample['y']**2)
    print 'Measured rms amplitude is %fV.' % sample['R']
    return sample
