"""
Zurich Instruments LabOne Python API Example

Demonstrate how to connect to a Zurich Instruments device via the Data Server
and perform some basic device configuration.
"""

# Copyright 2015 Zurich Instruments AG

import time
import re
import numpy as np

import zhinst.utils as utils
import zhinst.ziPython as ziPython


def run_example(device_id, amplitude=0.100):
    """
    Run the example: Connect to a Zurich Instruments device via the Data Server,
    configure some settings on the instrument, obtain a single demodulator
    sample via getSample(), calculate and add its RMS amplitude as a field to
    the ``sample`` dictionary and return that dictionary.

    Note:

      This is intended to be a simple example demonstrating how to connect to a
      Zurich Instruments device from ziPython. In most cases, data acquisition
      should use either ziDAQServer's poll() method or an instance of the
      ziDAQRecorder class, not the getSample() method.

    Requirements:

      Hardware configuration: Connect signal output 1 to signal input 1 with a
      BNC cable.

    Arguments:

      device_id (str): The ID of the device to run the example with. For
        example, `dev2006` or `uhf-dev2006`.

      amplitude (float, optional): The amplitude to set on the signal output.

    Returns:

      sample (dict): The demodulator sample dictionary.

    Raises:

      RuntimeError: If the device is not connected to the Data Server.
    """
    # Create an instance of the ziDiscovery class.
    d = ziPython.ziDiscovery()

    # Determine the device identifier from it's ID.
    device = d.find(device_id).lower()

    # Get the device's default connectivity properties.
    props = d.get(device)

    # The maximum API level supported by this example.
    apilevel_example = 5
    # The maximum API level supported by the device class, e.g., MF.
    apilevel_device = props['apilevel']
    # Ensure we run the example using a supported API level.
    apilevel = min(apilevel_device, apilevel_example)
    # See the LabOne Programming Manual for an explanation of API levels.

    # Create a connection to a Zurich Instruments Data Server (an API session)
    # using the device's default connectivity properties.
    daq = ziPython.ziDAQServer(props['serveraddress'], props['serverport'], apilevel)

    # Check that the device is visible to the Data Server
    if device not in utils.devices(daq):
        raise RuntimeError("The specified device `%s` is not visible to the Data Server, " % device_id + \
                               "please ensure the device is connected by using the LabOne User Interface " + \
                               "or ziControl (HF2 Instruments).")

    # Find out whether the device is an HF2 or a UHF
    devtype = daq.getByte('/%s/features/devtype' % device)
    options = daq.getByte('/%s/features/options' % device)

    # Configure the device
    out_c = 0
    in_c = 0
    demod_c = 0
    osc_c = 0
    demod_rate = 10e3
    time_constant = 1e-6
    if re.match('UHF', devtype) and (not re.match('MF', options)):
        out_mixer_c = 3
    elif re.match('HF2LI', devtype) and (not re.match('MF', options)):
        out_mixer_c = 6
    elif re.match('MFLI', devtype) and (not re.match('MD', options)):
        out_mixer_c = 1
    else:
        out_mixer_c = 0

    # Create a base configuration: disable all outputs, demods and scopes
    general_setting = [['/%s/demods/*/rate' % device, 0],
                       ['/%s/demods/*/trigger' % device, 0],
                       ['/%s/sigouts/*/enables/*' % device, 0]]
    if re.match('HF2', devtype):
        general_setting.append(['/%s/scopes/*/trigchannel' % device, -1])
    else:  # UHFLI
        general_setting.append(['/%s/demods/*/enable' % device, 0])
        general_setting.append(['/%s/scopes/*/enable' % device, 0])

    daq.set(general_setting)

    # Now set the settings relevant to this experiment
    exp_setting = [['/%s/sigins/%d/ac'             % (device, in_c), 0],
                   ['/%s/sigins/%d/range'          % (device, in_c), 2*amplitude],
                   ['/%s/demods/%d/order'          % (device, demod_c), 4],
                   ['/%s/demods/%d/timeconstant'   % (device, demod_c), time_constant],
                   ['/%s/demods/%d/rate'           % (device, demod_c), demod_rate],
                   ['/%s/demods/%d/harmonic'       % (device, demod_c), 1],
                   ['/%s/oscs/%d/freq'             % (device, osc_c), 400e3],
                   ['/%s/sigouts/%d/on'            % (device, out_c), 1],
                   ['/%s/sigouts/%d/enables/%d'    % (device, out_c, out_mixer_c), 1],
                   ['/%s/sigouts/%d/range'         % (device, out_c), 1],
                   ['/%s/sigouts/%d/amplitudes/%d' % (device, out_c, out_mixer_c), amplitude]]
    # Device class dependent configuration
    if re.match('HF2', devtype):
        exp_setting.append(['/%s/demods/%d/oscselect' % (device, demod_c), osc_c])
        exp_setting.append(['/%s/demods/%d/adcselect' % (device, demod_c), in_c])
        exp_setting.append(['/%s/sigins/%d/diff'      % (device, in_c), 0])
        exp_setting.append(['/%s/sigouts/%d/add'      % (device, out_c), 0])
    else:  # UHF
        exp_setting.append(['/%s/demods/%d/enable' % (device, demod_c), 1])
        if re.match('.*MF', options):
            exp_setting.append(['/%s/demods/%d/oscselect' % (device, demod_c), osc_c])
            exp_setting.append(['/%s/demods/%d/adcselect' % (device, demod_c), in_c])
    daq.set(exp_setting)

    # Wait for the demodulator filter to settle
    time.sleep(10*time_constant)

    # Perform a global synchronisation between the device and the data server:
    # Ensure that the settings have taken effect on the device before issuing
    # the getSample() command. Note: the sync() must be issued after waiting for
    # the demodulator filter to settle above.
    daq.sync()

    # Obtain one demodulator sample via ziDAQServer's low-level getSample()
    # method - for extended data acquisition it's preferable to use
    # ziDAQServer's poll() method or the ziDAQRecorder class.
    sample = daq.getSample('/%s/demods/0/sample' % device)
    sample['R'] = np.sqrt(sample['x']**2 + sample['y']**2)
    print 'Measured rms amplitude is %fV.' % sample['R']
    return sample
