"""
Zurich Instruments LabOne Python API Example

Demonstrate how to save and load Zurich Instruments device settings
asynchronously using the ziDeviceSettings class.

Note: This example is intended for experienced users who require a non-blocking
(asynchronous) interface for loading and saving settings. In general, the
utility functions save_settings() and load_settings() are more appropriate; see
`example_save_device_settings_simple'.
"""

# Copyright 2015 Zurich Instruments AG

import time
import os

import zhinst.utils as utils
import zhinst.ziPython as ziPython


def run_example(device_id, settings_file_path=None):
    """
    Run the example: Connect to a Zurich Instruments instrument, save the
    instrument's settings to file, toggle the signal output enable and reload
    the settings file.

    Note: This example is intended for experienced users who require a
    non-blocking (asynchronous) interface for loading and saving settings. In
    general, the utility functions save_settings() and load_settings() are more
    appropriate; see `example_save_device_settings_simple'.

    Arguments:

      device_id (str): The ID of the device to run the example with. For
        example, `dev2006` or `uhf-dev2006`.

      setting_file_path (str, optional): Specify the path where to save the
        settings file.

    Raises:

      RuntimeError: If the device is not connected to the Data Server.
    """
    # Create an instance of the ziDiscovery class.
    d = ziPython.ziDiscovery()

    # Determine the device identifier from it's ID.
    device = d.find(device_id).lower()

    # Get the device's default connectivity properties.
    props = d.get(device)

    # The maximum API level supported by this example.
    apilevel_example = 5
    # The maximum API level supported by the device class, e.g., MF.
    apilevel_device = props['apilevel']
    # Ensure we run the example using a supported API level.
    apilevel = min(apilevel_device, apilevel_example)
    # See the LabOne Programming Manual for an explanation of API levels.

    # Create a connection to a Zurich Instruments Data Server (an API session)
    # using the device's default connectivity properties.
    daq = ziPython.ziDAQServer(props['serveraddress'], props['serverport'], apilevel)

    # Check that the device is visible to the Data Server
    if device not in utils.devices(daq):
        raise RuntimeError("The specified device `%s` is not visible to the Data Server, " % device_id + \
                               "please ensure the device is connected by using the LabOne User Interface " + \
                               "or ziControl (HF2 Instruments).")

    timestr = time.strftime("%Y%m%d_%H%M")
    filename_noext = timestr + '_settings'  # Change this to the filename you want to save.

    timeout = 500  # [ms]
    device_settings = daq.deviceSettings(timeout)
    device_settings.set('devicesettings/device', device)
    device_settings.set('devicesettings/filename', filename_noext)
    if settings_file_path:
        device_settings.set('devicesettings/path', settings_file_path)
    # Set the path to '.' save to the current directory.
    # device_settings.set('devicesettings/path', '.')
    # NOTE: in this case, this example will have to be executed from a folder
    # where you have write access.

    toggle_device_setting(daq, device)

    # Save settings
    print "Saving settings..."
    device_settings.set('devicesettings/command', 'save')
    device_settings.execute()
    while not device_settings.finished():
        time.sleep(0.2)
    print "Done."

    data = device_settings.get('devicesettings/path')
    path = data['path'][0]
    filename_full_path = os.path.join(path, filename_noext) + '.xml'
    assert os.path.isfile(filename_full_path), "Failed to save settings file '%s'" % filename_full_path
    print "Saved file '%s'." % filename_full_path

    toggle_device_setting(daq, device)

    # Load settings
    print "Loading settings..."
    device_settings.set('devicesettings/command', 'save')
    device_settings.execute()
    while not device_settings.finished():
        time.sleep(0.2)
    print "Done."
    device_settings.clear()


def toggle_device_setting(daq, device):
    """
    Toggle a setting on the device: If it's enabled, disable the setting, and
    vice versa.
    """
    path = '/%s/sigouts/0/on' % device
    is_enabled = daq.getInt(path)
    print "Toggling setting '%s'." % path
    daq.setInt(path, not is_enabled)
    daq.sync()
