"""
Zurich Instruments LabOne Python API Example

Demonstrate how to perform an FFT on demodulator data the ziDAQZoomFFT
class/module.
"""

# Copyright 2015 Zurich Instruments AG

import re
import time
import numpy as np

import zhinst.utils as utils
import zhinst.ziPython as ziPython


def run_example(device_id, amplitude=0.1, do_plot=False):
    """
    Run the example: Perform a zoom FFT using ziPython's ziDAQZoomFFT module.

    Requirements:

      Hardware configuration: Connect signal output 1 to signal input 1 with a
      BNC cable.

    Arguments:

      device_id (str): The ID of the device to run the example with. For
        example, `dev2006` or `uhf-dev2006`.

      amplitude (float, optional): The amplitude to set on the signal output.

      do_plot (bool, optional): Specify whether to plot the zoomFFT. Default is
        no plot output.

    Returns:

      sample (list of dict): A list of demodulator sample dictionaries. Each
        entry in the list correspond to the result of a single zoomFFT.

    Raises:

      RuntimeError: If the device is not connected to the Data Server.
    """
    # Create an instance of the ziDiscovery class.
    d = ziPython.ziDiscovery()

    # Determine the device identifier from it's ID.
    device = d.find(device_id).lower()

    # Get the device's default connectivity properties.
    props = d.get(device)

    # The maximum API level supported by this example.
    apilevel_example = 5
    # The maximum API level supported by the device class, e.g., MF.
    apilevel_device = props['apilevel']
    # Ensure we run the example using a supported API level.
    apilevel = min(apilevel_device, apilevel_example)
    # See the LabOne Programming Manual for an explanation of API levels.

    # Create a connection to a Zurich Instruments Data Server (an API session)
    # using the device's default connectivity properties.
    daq = ziPython.ziDAQServer(props['serveraddress'], props['serverport'], apilevel)

    # Check that the device is visible to the Data Server
    if device not in utils.devices(daq):
        raise RuntimeError("The specified device `%s` is not visible to the Data Server, " % device_id + \
                               "please ensure the device is connected by using the LabOne User Interface " + \
                               "or ziControl (HF2 Instruments).")

    # Find out whether the device is an HF2 or a UHF
    devtype = daq.getByte('/%s/features/devtype' % device)
    options = daq.getByte('/%s/features/options' % device)

    # Configure the device
    out_c = 0
    in_c = 0
    demod_c = 0
    osc_c = 0
    demod_rate = 10e3
    time_constant = 8e-5
    if re.match('UHF', devtype) and (not re.match('MF', options)):
        out_mixer_c = 3
    elif re.match('HF2LI', devtype) and (not re.match('MF', options)):
        out_mixer_c = 6
    elif re.match('MFLI', devtype) and (not re.match('MD', options)):
        out_mixer_c = 1
    else:
        out_mixer_c = 0

    # create a base configuration: disable all outputs, demods and scopes
    general_setting = [
        ['/%s/demods/*/rate' % device, 0],
        ['/%s/demods/*/trigger' % device, 0],
        ['/%s/sigouts/*/enables/*' % device, 0]]

    # if the device is a UHF additionally disable all demodulators
    if re.match('UHF', devtype):
        general_setting.append(['/%s/demods/*/enable' % device, 0])
        general_setting.append(['/%s/scopes/*/enable' % device, 0])
    elif re.match('HF2', devtype):
        general_setting.append(['/%s/scopes/*/trigchannel' % device, -1])
    daq.set(general_setting)

    # now set the settings relevant to this experiment
    exp_setting = [
        ['/%s/sigins/%d/ac'             % (device, in_c), 0],
        ['/%s/sigins/%d/range'          % (device, in_c), 2*amplitude],
        ['/%s/demods/%d/order'          % (device, demod_c), 4],
        ['/%s/demods/%d/rate'           % (device, demod_c), demod_rate],
        ['/%s/demods/%d/timeconstant'   % (device, demod_c), time_constant],
        ['/%s/demods/%d/harmonic'       % (device, demod_c), 1],
        ['/%s/demods/%d/phaseshift'     % (device, demod_c), 0],
        ['/%s/oscs/%d/freq'             % (device, demod_c), 400e3],
        ['/%s/sigouts/%d/on'            % (device, out_c), 1],
        ['/%s/sigouts/%d/range'         % (device, out_c), 1],
        ['/%s/sigouts/%d/enables/%d'    % (device, out_c, out_mixer_c), 1],
        ['/%s/sigouts/%d/amplitudes/%d' % (device, out_c, out_mixer_c), amplitude]]

    # device dependent configuration
    if re.match('UHF', devtype):
        exp_setting.append(['/%s/sigins/%d/imp50' % (device, in_c), 1])
        exp_setting.append(['/%s/demods/%d/enable' % (device, demod_c), 1])
        if re.match('.*MF', options):
            exp_setting.append(['/%s/demods/%d/oscselect' % (device, demod_c), osc_c])
            exp_setting.append(['/%s/demods/%d/adcselect' % (device, demod_c), in_c])
    elif re.match('HF2', devtype):
        exp_setting.append(['/%s/demods/%d/oscselect' % (device, demod_c), osc_c])
        exp_setting.append(['/%s/demods/%d/adcselect' % (device, demod_c), in_c])
        exp_setting.append(['/%s/sigins/%d/diff' % (device, in_c), 0])
        exp_setting.append(['/%s/sigouts/%d/add'  % (device, out_c), 0])
    daq.set(exp_setting)

    # Wait for the demodulator filter to settle
    time.sleep(10*time_constant)

    # Create a thread for an instance of the zoomFFT class
    timeout = 500
    zoomfft = daq.zoomFFT(timeout)
    # set zoomFFT parameters
    # the device on which the zoomFFT will be performed
    zoomfft.set('zoomFFT/device', device)
    # select standard FFT(x + iy)
    zoomfft.set('zoomFFT/mode', 0)
    # disable overlap mode
    zoomfft.set('zoomFFT/overlap', 0)
    # return absolute frequency
    zoomfft.set('zoomFFT/absolute', 1)
    # number of lines 2^bits
    zoomfft.set('zoomFFT/bit', 16)
    # number of zoomFFT's to perform
    loopcount = 1
    zoomfft.set('zoomFFT/loopcount', loopcount)

    # subscribe to the nodes from which data will be recorded
    path = '/%s/demods/%d/sample' % (device, demod_c)
    zoomfft.subscribe(path)

    # start the zoomFFT
    zoomfft.execute()

    start = time.time()
    timeout = 30  # [s]
    print "Will perform %d zoomFFTs" % loopcount
    while not zoomfft.finished():
        time.sleep(0.2)
        # Please note: progress() and finish() works for first zoomFFT, but
        # It's a known issue that it doesn't update for subsequent zoomFFTs
        progress = zoomfft.progress()
        print "Individual zoomFFT %.2f%% complete.\r" % (100*progress),
        ## here we could read intermediate data via
        # data = zoomfft.read()...
        ## and process it
        # if device in data:
        # ...
        if (time.time() - start) > timeout:
            # if for some reason the zoomFFT is blocking force the end of the
            # measurement
            print "\nzoomFFT still not finished, forcing finish..."
            zoomfft.finish()
    print ""

    # Read the zoomFFT data. this command can also be executed whilst the
    # zoomFFT is still being calculated (before finished() is True), in this
    # case zoomFFT data up to that time point is returned. it's still
    # necessary to issue read() at the end to fetch the rest.
    return_flat_data_dict = True
    data = zoomfft.read(return_flat_data_dict)
    zoomfft.unsubscribe(path)

    # Stop the sweeper thread and clear the memory
    zoomfft.clear()

    # Check the dictionary returned is non-empty
    assert data, "read() returned an empty data dictionary, did you subscribe to any paths?"
    # Note: data could be empty if no data arrived, e.g., if the demods were
    # disabled or had rate 0
    assert path in data, "data dictionary has no key '%s'" % path
    samples = data[path]
    num_zoomfft = len(samples)
    print "sample contains %d zoomFFTs" % num_zoomfft
    # please note: the "[0][0]" indexing is known issue to be fixed in an
    # upcoming release
    print "Number of lines in first zoomFFT: %d" % len(samples[0][0]['grid'])

    if do_plot:
        plot_idx = num_zoomfft - 1
        print "Will plot result of zoomFFT sample[%d]" % plot_idx
        frequencies = samples[plot_idx][0]['grid']
        r = samples[plot_idx][0]['r']
        filter_data = samples[plot_idx][0]['filter']
        import matplotlib.pyplot as plt
        plt.figure()
        plt.subplot(211)
        plt.hold(True)
        plt.title('Results of zoomFFT')
        plt.plot(frequencies, 20*np.log10(r*2*np.sqrt(2)/amplitude))
        plt.grid(True)
        plt.xlabel('Frequency (Hz)')
        plt.ylabel('Amplitude (dBV)')
        plt.autoscale(True, 'both', True)
        plt.subplot(212)
        plt.hold(True)
        plt.plot(frequencies, 20*np.log10((r/filter_data)*2*np.sqrt(2)/amplitude))
        plt.grid(True)
        plt.xlabel('Frequency (Hz)')
        plt.ylabel('Amplitude (dBV) scaled with Filter')
        plt.autoscale(True, 'both', True)
        plt.draw()
        plt.show()

    return samples
