"""
Zurich Instruments LabOne Python API Example

Demonstrate how to obtain scope data from an HF2's signal outputs using
ziDAQServer's blocking (synchronous) poll() command.
"""

# Copyright 2015  Zurich Instruments AG

import re
import warnings
import numpy as np

import zhinst.utils as utils
import zhinst.ziPython as ziPython


def run_example(device_id, amplitude=0.5, sigouts_range=1.0, do_plot=False):
    """
    Run the example: Connect to a Zurich Instruments device via the Data Server,
    generate a sine wave on the signal outputs and obtain the waveform from the
    signal outputs by polling data from the scope using ziDAQServer's blocking
    (synchronous) poll() command.

    Note: This is an HF2 example and uses API level 1; users of other device
      classes are recommended to connect via API level 4, particularly when
      obtaining scope data.

    Arguments:

      amplitude (float, optional): The amplitude to set on the signal output.

      sigouts_range (float, optional): The range to use on the signal output.

      do_plot (bool, optional): Specify whether to plot the polled data. Default
        is no plot output.

    Returns:

      shots (list of dict): A list containing the polled scope shots, each entry
        is a dictionary holding the various data associated with a shot.

    Raises:

      RuntimeError: If the device is not connected to the Data Server.
    """
    # Create an instance of the ziDiscovery class.
    d = ziPython.ziDiscovery()

    # Determine the device identifier from it's ID.
    device = d.find(device_id).lower()

    # Get the device's default connectivity properties.
    props = d.get(device)

    # The maximum API level supported by this example.
    apilevel_example = 1
    # The maximum API level supported by the device class, e.g., MF.
    apilevel_device = props['apilevel']
    # Ensure we run the example using a supported API level.
    apilevel = min(apilevel_device, apilevel_example)
    # See the LabOne Programming Manual for an explanation of API levels.

    # Create a connection to a Zurich Instruments Data Server (an API session)
    # using the device's default connectivity properties.
    daq = ziPython.ziDAQServer(props['serveraddress'], props['serverport'], apilevel)

    # Check that the device is visible to the Data Server
    if device not in utils.devices(daq):
        raise RuntimeError("The specified device `%s` is not visible to the Data Server, " % device_id + \
                               "please ensure the device is connected by using the LabOne User Interface. " + \
                               "or ziControl (HF2 Instruments).")

    # Check the device class
    devtype = daq.getByte('/%s/features/devtype' % device)
    assert re.match('HF2', devtype), "This example is only for HF2 Instruments. " + \
        "Is your device really an HF2? Detected devtype: `%s`." % devtype

    options = daq.getByte('/%s/features/options' % device)
    # Define configuration and configure the device
    frequency = 1e6
    out_c = 0
    osc_c = 0
    if re.match('HF2LI', devtype) and (not re.match('MF', options)):
        out_mixer_c = 6
    else:
        out_mixer_c = 0

    # create a base configuration: disable all outputs, demods and scopes
    general_setting = [
        ['/%s/demods/*/rate'        % device, 0],
        ['/%s/demods/*/trigger'     % device, 0],
        ['/%s/sigouts/*/enables/*'  % device, 0],
        ['/%s/scopes/*/trigchannel' % device, -1]]
    daq.set(general_setting)

    # The scope's sampling rate is configured by specifying the ``time`` node
    # (/devN/scopes/0/time). The rate is equal to 210e6/2**time, where 210e6 is
    # the HF2 ADC's sampling rate (^=clockbase, /devX/clockbase). time is an
    # integer in range(0,16).
    #
    # Since the length of a scope shot is fixed (2048) on an HF2, specifying the
    # rate also specifies the time duration of a scope shot,
    # t_shot=2048*1./rate=2048*2**time/210e6.
    #
    # Therefore, if we would like to obtain (at least) 10 periods of the signal
    # generated by Oscillator 1, we need to set the scope's time parameter as
    # following:
    clockbase = float(daq.getInt('/%s/clockbase' % device))  # 210e6 for HF2
    desired_t_shot = 10./frequency
    scope_time = np.ceil(np.max([0, np.log2(clockbase*desired_t_shot/2048.)]))
    if scope_time > 15:
        scope_time = 15
        warnings.warn("Can't not obtain scope durations of %.3f s, scope shot duration will be %.3f." \
                          % (desired_t_shot, 2048.*2**scope_time/clockbase))
    print "Will set /%s/scopes/0/time to %d." % (device, scope_time)
    # Now set the settings relevant to this experiment. Note: Since we're taking
    # scope data from the signal outputs, we don't configure the signal inputs.
    exp_setting = [
        ['/%s/oscs/%d/freq'             % (device, osc_c), frequency],
        ['/%s/sigouts/%d/on'            % (device, out_c), 1],
        ['/%s/sigouts/%d/add'           % (device, out_c), 0],
        ['/%s/sigouts/%d/enables/%d'    % (device, out_c, out_mixer_c), 1],
        ['/%s/sigouts/%d/range'         % (device, out_c), sigouts_range],
        ['/%s/sigouts/%d/amplitudes/%d' % (device, out_c, out_mixer_c), amplitude/sigouts_range],
        ['/%s/scopes/0/channel'         % (device), 2],  # 2 ^= Signal Output 1
        ['/%s/scopes/0/triglevel'       % (device), 0.0],
        ['/%s/scopes/0/trigholdoff'     % (device), 0.1],
        # Enable bandwidth limiting: avoid antialiasing effects due to
        # subsampling when the scope sample rate is less than the input
        # channel's sample rate.
        ['/%s/scopes/0/bwlimit'         % (device), 1],
        ['/%s/scopes/0/time'            % (device), scope_time],  # set the sampling rate
        ['/%s/scopes/0/trigchannel'     % (device), 2]]
    daq.set(exp_setting)

    # Perform a global synchronisation between the device and the data server:
    # Ensure that 1. the settings have taken effect on the device before issuing
    # the poll() command and 2. clear the API's data buffers.
    daq.sync()

    # Subscribe to the scope data
    path = '/%s/scopes/0/wave' % (device)
    daq.subscribe(path)

    # Poll data for 1s
    poll_length = 1  # [s]
    poll_timeout = 500  # [ms]
    poll_flags = 0
    poll_return_flat_dict = True
    data = daq.poll(poll_length, poll_timeout, poll_flags, poll_return_flat_dict)

    # Unsubscribe from all paths
    daq.unsubscribe('*')
    # Disable the scope
    daq.setInt('/%s/scopes/0/trigchannel' % device, -1)

    # Check the dictionary returned is non-empty
    assert data, "poll() returned an empty data dictionary, scope's trigger criteria not violated?"
    # Note, the data could be empty if no data arrived, e.g., if the scope was
    # disabled
    assert path in data, "data dictionary has no key '%s'" % path
    # The data returned by poll is a dictionary whose keys correspond to the
    # subscribed node paths. Looking up a key in the dictionary returns the data
    # associated with that node.
    shots = data[path]
    # The scope data polled from the node /devN/scopes/0/wave, here ``shots``,
    # is a list of dictionaries; the length of ``shots`` is the number of scope
    # shots that were returned by poll().
    print "poll() returned %i scope shots." % len(shots)
    assert len(shots) >= 0, "len(data[%s]) is 0: no scope shots." % path
    # The values of the scope shot are 16-bit integers
    min_val = np.min(shots[0]['wave'])
    max_val = np.max(shots[0]['wave'])
    # In order to obtain the physical value of the wave we need to scale
    # accordingly:
    sigouts_range_set = daq.getDouble('/%s/sigouts/0/range' % device)
    scale = sigouts_range_set/(2**15)  # The scope's wave are 16-bit integers
    print "Maximum value of first shot: %+d (int16), scaled: %+.3f V" % (min_val, scale*min_val)
    print "Minimum value of first shot: %+d (int16), scaled: %+.3f V" % (max_val, scale*max_val)

    if do_plot:
        import matplotlib.pyplot as plt
        # Create plot
        plt.figure()
        plt.grid(True)
        for shot in shots:
            t = np.linspace(0, shot['dt']*len(shot['wave']), len(shot['wave']))
            plt.plot(t, scale*shot['wave'])
        plt.title('Scope shots (num_shots=%d)' % len(shots))
        plt.xlabel('Time (s)')
        plt.ylabel('Amplitude (V)')
        plt.ylim([-sigouts_range_set, sigouts_range_set])
        plt.draw()
        plt.show()

    return shots
